-- Copyright (c) 2015 SameGoal, Inc.
-- All Rights Reserved.

-- DEFAULT STORED PROCEDURES FILE
-- TODO(user): Customize per district needs.

-- ============================================================================
-- 1. MAPPING TABLES
--
-- Mapping tables map between documentID in SameGoal and documentID
-- (or equivalent) in your third-party system. By tracking documents added
-- to your third-party system via SameGoal integration in a mapping table,
-- diff handling stored procedures can easily identify those which should be
-- considered during integration (and ignore earlier records). Also,
-- changes to other tables in your third-party system that use the
-- third-party documentID can be updated by mapping through the SameGoal
-- documentID.
--
-- TODO(user): Add stored procedures to create mapping tables if they are not
-- yet installed. We suggest you create a mapping table for each type of
-- documentID found in your system, and prefix it with "sg". Below, find an
-- an example for a system which uses a "Plan" and "Eval" table internally.
-- ============================================================================

IF OBJECT_ID('sgPlan') IS NULL
   CREATE TABLE [sgPlan] (
     [planID] INT IDENTITY NOT NULL PRIMARY KEY,
     [SG_docID] VARCHAR(50) NOT NULL,
     CONSTRAINT unique_plan_docID UNIQUE(SG_docID)
   );

IF OBJECT_ID('sgEval') IS NULL
   CREATE TABLE [sgEval] (
     [evalID] INT IDENTITY NOT NULL PRIMARY KEY,
     [SG_docID] VARCHAR(50) NOT NULL,
     CONSTRAINT unique_eval_docID UNIQUE(SG_docID)
   );

-- ============================================================================
-- 2. DIFF HANDLING STORED PROCEDURES
--
-- "Diff handling" stored procedures are called when the samegoal-integrator
-- program compares the data in a file extracted from SameGoal with a
-- same-named data file extracted from your third-party system. Traversing
-- row by row, samegoal-integrator compares the row in your SameGoal data file
-- with the next uninspected row in your third-party system data file.
-- The following pattern of cases emerge (example):
--
-- CASE A:
--   * Row appears in Plan.csv extracted from SameGoal.
--   * Row appears in Plan.csv extracted from your third-party system.
--   * All data in both rows is equivalent.
--   * OUTCOME: samegoal-integrator continues, no stored procedure is called.
--
-- CASE B:
--   * Row appears in Plan.csv extracted from SameGoal.
--   * Row does not appear in Plan.csv extracted from your third-party system.
--   * OUTCOME: samegoal-integrator calls stored procedure "sgPlanUpsert".
--
-- CASE C:
--   * Row appears in Plan.csv extracted from SameGoal.
--   * Row appears in Plan.csv extracted from your third-party system.
--   * Data in both rows differs.
--   * OUTCOME: samegoal-integrator calls stored procedure "sgPlanUpsert".
--
-- CASE D:
--   * Row does not appear in Plan.csv extracted from SameGoal.
--   * Row appears in Plan.csv extracted from your third-party system.
--   * OUTCOME: samegoal-integrator calls stored procedure "sgPlanDelete".
--
-- Thus, for every table updated in your third-party system, there should
-- be one "sgTableDelete" and one "sgTableUpsert" stored procedure defined.
--
-- TODO(user): Define "delete" and "upsert" stored procedures for each
-- third-party system table involved in the integration. These stored
-- procedures should limit activity to records tracked in the integration
-- mapping tables.
--
-- An example for a third-party system table called "Plan" is provided.
-- ============================================================================

-- === Plan (delete) ===
GO
IF OBJECT_ID('sgPlanDelete') IS NULL
  EXEC('CREATE PROCEDURE sgPlanDelete AS SELECT 1');
GO

ALTER PROCEDURE sgPlanDelete
  @SG_docID VARCHAR(50)
AS
BEGIN
  SET NOCOUNT ON;
  SET XACT_ABORT ON;
  BEGIN TRANSACTION;

  -- error on illegal inputs
  IF (@SG_docID IS NULL)
  BEGIN
    RAISERROR ('SG_docID cannot be NULL', 16, -1);
    ROLLBACK;
    RETURN;
  END

  -- lookup planID
  DECLARE @planID INT;
  SELECT @planID = [planID]
  FROM [sgPlan]
  WHERE [SG_docID] = @SG_docID;
  IF (@planID IS NULL)
  BEGIN
    RAISERROR ('planID cannot be NULL', 16, -1);
    ROLLBACK;
    RETURN;
  END

  -- delete map entry
  DELETE FROM [sgPlan] WHERE [SG_docID] = @SG_docID;

  -- delete row
  DELETE FROM [Plan] WHERE [planID] = @planID;
  COMMIT;
END;

-- === Plan (upsert) ===
GO
IF OBJECT_ID('sgPlanUpsert') IS NULL
  EXEC('CREATE PROCEDURE sgPlanUpsert AS SELECT 1');
GO

ALTER PROCEDURE sgPlanUpsert
  @SG_docID VARCHAR(50),
  @startDate SMALLDATETIME,

  -- SQLExtraCols
  @districtName VARCHAR(50),
  @username VARCHAR(50)
AS
BEGIN
  SET NOCOUNT ON;
  SET XACT_ABORT ON;
  BEGIN TRANSACTION;

  -- error on illegal inputs
  IF (@SG_docID IS NULL)
  BEGIN
    RAISERROR ('SG_docID cannot be NULL', 16, -1);
    ROLLBACK;
    RETURN;
  END

  -- lookup userID
  DECLARE @userID INT;
  SELECT @userID = [userID] FROM [Users] WHERE [username] = @username;
  IF (@userID IS NULL)
  BEGIN
    RAISERROR ('unable to find userID', 16, -1);
    ROLLBACK;
    RETURN;
  END

  -- lookup planID
  DECLARE @planID INT;
  SELECT @planID = [planID]
  FROM [sgPlan]
  WHERE [SG_docID] = @SG_docID;

  -- update existing row
  IF (@planID IS NOT NULL)
  BEGIN
    UPDATE [Plan]
    SET [startDate] = @startDate,
        [modifiedDate] = CURRENT_TIMESTAMP,
        [modifiedBy] = @userID
    WHERE [planID]= @planID;
    COMMIT;
    RETURN;
  END

  -- lookup districtID
  DECLARE @districtID INT;
  SELECT @districtID = [districtID]
  FROM [Districts]
  WHERE [name] = @districtName;
  IF (@districtID IS NULL)
  BEGIN
    RAISERROR ('unable to find districtID', 16, -1);
    ROLLBACK;
    RETURN;
  END

  -- insert new row
  INSERT [Plan] (
    [personID],
    [districtID],
    [startDate],
    [modifiedDate],
    [modifiedBy],
    [createdDate],
    [createdByID]
  ) VALUES (
    @personID,
    @districtID,
    @startDate,
    CURRENT_TIMESTAMP,
    @userID,
    CURRENT_TIMESTAMP,
    @userID
  );

  SELECT @planID = SCOPE_IDENTITY();
  IF (@planID IS NULL)
  BEGIN
    RAISERROR ('planID cannot be NULL', 16, -1);
    ROLLBACK;
    RETURN;
  END

  -- update mapping
  INSERT [sgPlan] (
    [SG_docID],
    [planID]
  ) VALUES (
    @SG_docID,
    @planID
  );
  COMMIT;
END;

-- ============================================================================
-- 3. FINAL STORED PROCEDURES
--
-- Final stored procedures are called from the "Upload" section of config.xml
-- after diff handling stored procedures have completed. Commonly, these
-- stored procedures are used to end-date/close any open records which
-- have a more current record inserted during diff handling.
--
-- TODO(user): Define final stored procedures which must be run. Each stored
-- procedure should be invoked from the "Upload" section of config.xml.
--
-- A simple example to end-date records in a third-party system table
-- called "Plan" is provided.
-- ============================================================================

GO
IF OBJECT_ID('sgPlan') IS NULL
  EXEC('CREATE PROCEDURE sgPlanEndDate AS SELECT 1');
GO

ALTER PROCEDURE sgPlanEndDate
AS
BEGIN
  SET NOCOUNT ON;
  SET XACT_ABORT ON;
  BEGIN TRANSACTION;

  UPDATE [Plan]
  SET [endDate] = pd.[endDate]
  FROM [PlanDates] AS pd
  WHERE [planID] = pd.planID;
  COMMIT;
END;
